import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:calculator_cli/src/commands/commands.dart';
import 'package:calculator_cli/src/core/exist_codes.dart';
import 'package:mason_logger/mason_logger.dart';

class CalculatorRunner extends CommandRunner<int> {
  CalculatorRunner() : super('calculator_cli', 'A Simple Calculator') {
    argParser.addFlag(
      'verbose',
      help: 'Noisy logging, including all shell commands executed.',
    );
    addCommand(AddCommand(logger: _logger));
    addCommand(SubtractCommand(logger: _logger));
    addCommand(DivideCommand(logger: _logger));
    addCommand(MultiplyCommand(logger: _logger));
    addCommand(AreaRunner(logger: _logger));
  }

  final Logger _logger = Logger(level: Level.info);

  @override
  Future<int> run(Iterable<String> args) async {
    try {
      final topLevelResults = parse(args);
      if (topLevelResults['verbose'] == true) {
        _logger.level = Level.verbose;
      }
      return await runCommand(topLevelResults) ?? ExitCodes.errors;
    } catch (e) {
      _logger.err('Runner throw $e');
      _logger.err('CLI was unable to process the command');
      return ExitCodes.errors;
    }
  }

  @override
  Future<int?> runCommand(ArgResults topLevelResults) async {
    return await super.runCommand(topLevelResults);
  }
}
